import time

from pytagsfs.specialfile.logfile import VirtualLogFile
from pytagsfs.debug import log_critical, log_info
from pytagsfs.util import wraps, join_path_abs


_profiling_enabled = False

def enable_profiling():
    log_info('enabling profiling; times are in ms')
    global _profiling_enabled
    _profiling_enabled = True


def disable_profiling():
    log_info('disabling profiling')
    global _profiling_enabled
    _profiling_enabled = False


def profile(fn, *args, **kwargs):
    start = int(1000.0 * time.time())
    try:
        return fn(*args, **kwargs)
    finally:
        duration = int(1000.0 * time.time()) - start
        log_critical(u'PROF %s %s', duration, fn.__name__)


def profiled(fn):
    @wraps(fn)
    def wrapped(*args, **kwargs):
        if not _profiling_enabled:
            return fn(*args, **kwargs)
        return profile(fn, *args, **kwargs)
    return wrapped
